from flask import render_template, Blueprint, current_app, g
from flask import request, after_this_request, redirect, session
from app import auth

from flask import jsonify
from flask_pyoidc.user_session import UserSession

from hsapi import Node, User, Route

import logging
log = logging.getLogger()

main_blueprint = Blueprint('main', __name__)


# @main_blueprint.before_request
# def set_theme():
#     g.theme = request.cookies.get('theme', 'light')


@main_blueprint.route('/', methods=['GET', 'POST'])
@auth.oidc_auth('default')
def index():
    user_session = UserSession(session)
    return jsonify(access_token=user_session.access_token,
                   id_token=user_session.id_token,
                   userinfo=user_session.userinfo)


@main_blueprint.route('/logout')
@auth.oidc_logout
def logout():
    return redirect('/')


@main_blueprint.route('/nodes', methods=['GET'])
def nodes():

    nodelist = Node().list()
    log.debug(nodelist)

    return render_template('nodes.html',
                           nodes=nodelist.nodes)


@main_blueprint.route('/node/<int:nodeId>', methods=['GET'])
def node(nodeId):
    # There is a bug in HS api with retrieving a single node
    # and we added a workaround to hsapi, so node.get() returns a
    # v1Node object instead of v1NodeResponse, so we access directly
    # `node`, instead of `node.node`
    node = Node().get(nodeId)
    routes = Node().routes(nodeId)
    isExitNode = any((r for r in routes.routes if r.prefix.endswith('/0')))
    log.debug(node)
    return render_template("node.html",
                           routes=routes.routes,
                           isExitNode=isExitNode,
                           node=node.node)


@main_blueprint.route('/users', methods=['GET'])
def users():

    userList = User().list()
    # Get online status of devices of the user
    online = {}
    nodeList = Node().list()
    for user in userList.users:
        log.debug(user)
        userNodeList = [n for n in nodeList.nodes if n.user.name == user.name]
        log.debug(userNodeList)
        online[user.name] = any(map(lambda x: x.online, userNodeList))

    return render_template('users.html',
                           users=userList.users,
                           online=online)


@main_blueprint.route('/user/<userName>', methods=['GET'])
def user(userName):
    # There is a bug in HS api with retrieving a single node
    # and we added a workaround to hsapi, so node.get() returns a
    # v1Node object instead of v1NodeResponse, so we access directly
    # `node`, instead of `node.node`
    user = User().get(userName)
    log.debug(user)
    userNodeList = [n for n in Node().list().nodes if n.user.name == userName]
    return render_template("user.html",
                           user=user.user,
                           userNodeList=userNodeList)


@main_blueprint.route('/routes', methods=['GET'])
def routes():
    routes = Route().list()
    log.debug(routes)

    prefixes = set(
        (r.prefix for r in routes.routes if not r.prefix.endswith('/0')))

    exitNodes = [r.node for r in routes.routes if r.prefix.endswith(('0/0'))]

    final = {}
    for prefix in prefixes:
        rrp = [x for x in routes.routes if x.prefix == prefix]
        final[prefix] = sorted(rrp, key=lambda x: x.isPrimary, reverse=True)
    log.debug(final)
    return render_template("routes.html",
                           exitNodes=exitNodes,
                           routes=final)
